/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Cluster.hpp
 *
 *  Created on: 2010/12/16
 *      Author: HARK Development Team
 */

#ifndef CLUSTER_HPP_
#define CLUSTER_HPP_

#include <vector>
#include <Band.hpp>
#include <BandAndPower.hpp>
#include <Logger.hpp>

namespace NSHarkBinaural
{

/**
 * 周波数バンドのクラスタリングに使うクラス.
 */
class Cluster
{
public:
    /**
     * 周波数バンドとパワースペクトルデータの組.
     */
    //typedef BandAndPower BandAndPower;

    /**
     * 空のクラスタを生成する.
     */
    Cluster();

    /**
     * 指定された周波数バンドを保持するクラスタを生成する.
     */
    Cluster(Band band, double power);

    /**
     * 指定されたクラスタを自身にマージする.
     * @param cluster このクラスタの中身を自身にマージする
     */
    void Merge(const Cluster& cluster);

    /**
     * クラスタに含まれるデータの個数を得る.
     * @return データの個数
     */
    size_t Size() const
    {
        return this->bandAndPowerList.size();
    }

    /**
     * クラスタに含まれるデータを得る.
     * @return 周波数バンドとそのバンドのパワーのリスト
     */
    const std::vector<BandAndPower>& BandAndPowerList() const
    {
        return this->bandAndPowerList;
    }

    /**
     * 指定されたクラスタとのコストを計算する.
     * 自分からclusterへのコストを計算する。
     * a.ContF(b)とb.CostF(a)は一般に等しくならないことに注意。
     * @param cluster コストを計算するクラスタ
     */
    double CostF(const Cluster& cluster);

private:
    /**
     * クラスタに含まれる周波数バンドと、そのバンドのパワースペクトル一覧.
     */
    std::vector<BandAndPower> bandAndPowerList;

    //double avg;     /* average of data */
    //double avgi;    /* average of band */
    //double varience; /* varience of data */

    //static Logger& logger;
};

}

#endif /* CLUSTER_HPP_ */
